@@ -33,6 +33,9 @@ import re
 import sys
 import tty
 import os
+import struct
+import termios
+import fcntl
 import getpass
 import cmd
 import traceback
@@ -90,7 +93,12 @@ class PwmanCli(cmd.Cmd):
         for i in idstrs:
             m = rx.match(i)
             if m == None:
-                ids.append(int(i))
+                try:
+                    ids.append(int(i))
+                except ValueError, e:
+                    self._db.clearfilter()
+                    self._db.filter([Tag(i)])
+                    ids += self._db.listnodes()
             else:
                 ids += range(int(m.group(1)),
                              int(m.group(2))+1)
@@ -107,7 +115,11 @@ class PwmanCli(cmd.Cmd):
         if len(password) == 0:
             length = getinput("Password length (default 7): ", "7")
             length = int(length)
-            (password, dumpme) = generator.generate_password(length, length)
+
+            numerics = config.get_value("Generator", "numerics") == 'true';
+            leetify = config.get_value("Generator", "leetify") == 'true';
+            (password, dumpme) = generator.generate_password(length, length, True, leetify, numerics)
+
             print "New password: %s" % (password)
             return password
         else:
@@ -352,10 +364,14 @@ class PwmanCli(cmd.Cmd):
         self.do_list(args)
         
     def do_list(self, args):
-        self.do_filter('')
+        if len(args.split()) > 0:
+            self.do_clear('')
+            self.do_filter(args)
         try:
+            rows, cols = gettermsize()
             nodeids = self._db.listnodes()
             nodes = self._db.getnodes(nodeids)
+            cols -= 8
             i = 0
             for n in nodes:
                 tags=n.get_tags()
@@ -368,15 +384,19 @@ class PwmanCli(cmd.Cmd):
                         first=False
                     tagstring += t.get_name()
                 name = "%s@%s" % (n.get_username(), n.get_url())
-                if len(name) > 30:
-                    name = name[:27] + "..."
-                if len(tagstring) > 20:
-                    tagstring = tagstring[:17] + "..."
-                    
-                print typeset("%5d. %-30s %-20s" % (n.get_id(), name, tagstring),
+
+                name_len = cols * 2 / 3
+                tagstring_len = cols / 3
+                if len(name) > name_len:
+                    name = name[:name_len-3] + "..."
+                if len(tagstring) > tagstring_len:
+                    tagstring = tagstring[:tagstring_len-3] + "..."
+
+                fmt = "%%5d. %%-%ds %%-%ds" % (name_len, tagstring_len)
+                print typeset(fmt % (n.get_id(), name, tagstring),
                               ANSI.Yellow, False)
                 i += 1
-                if i > 23:
+                if i > rows-2:
                     i = 0
                     c = getonechar("Press <Space> for more, or 'Q' to cancel")
                     if c == 'q':
@@ -448,14 +468,14 @@ class PwmanCli(cmd.Cmd):
         self.help_list()
         
     def help_list(self):
-        self.usage("list")
-        print "List nodes that match current filter. ls is an alias."
+        self.usage("list <tag> ...")
+        print "List nodes that match current or specified filter. ls is an alias."
 
     def help_EOF(self):
         self.help_exit()
 
     def help_delete(self):
-        self.usage("delete <ID> ...")
+        self.usage("delete <ID|tag> ...")
         print "Deletes nodes. rm is an alias."
         self._mult_id_help()
 
@@ -479,7 +499,7 @@ class PwmanCli(cmd.Cmd):
         self.help_list()
 
     def help_edit(self):
-        self.usage("edit <ID> ... ")
+        self.usage("edit <ID|tag> ... ")
         print "Edits a nodes."
         self._mult_id_help()
     
@@ -488,8 +508,8 @@ class PwmanCli(cmd.Cmd):
         print "Imports a nodes from a file."
 
     def help_export(self):
-        self.usage("export <ID> ... ")
-        print "Exports a list of ids to an external format. If no IDs are specified, then all nodes under the current filter are exported."
+        self.usage("export <ID|tag> ... ")
+        print "Exports a list of ids to an external format. If no IDs or tags are specified, then all nodes under the current filter are exported."
         self._mult_id_help()
         
     def help_new(self):
@@ -500,12 +520,12 @@ class PwmanCli(cmd.Cmd):
         self.help_delete()
     
     def help_print(self):
-        self.usage("print <ID> ...")
+        self.usage("print <ID|tag> ...")
         print "Displays a node. ",
         self._mult_id_help()
 
     def _mult_id_help(self):
-        print "Multiple ids can be specified, separated by a space. A range of ids can be specified in the format n-N. e.g. 10-20 would specify all ids from 10 to 20 inclusive."
+        print "Multiple ids and nodes can be specified, separated by a space. A range of ids can be specified in the format n-N. e.g. '10-20' would specify all nodes having ids from 10 to 20 inclusive. Tags are considered one-by-one. e.g. 'foo 2 bar' would yield to all nodes with tag 'foo', node 2 and all nodes with tag 'bar'."
     
     def help_exit(self):
         self.usage("exit")
@@ -606,6 +626,12 @@ def getyesno(question, defaultyes=False, width=_defaultwidth):
     else:
         return getyesno(question, defaultyes, width)
 
+def gettermsize():
+    s = struct.pack("HHHH", 0, 0, 0, 0)
+    f = sys.stdout.fileno()
+    x = fcntl.ioctl(f, termios.TIOCGWINSZ, s)
+    rows, cols, width, height = struct.unpack("HHHH", x)
+    return rows, cols
 
 def getinput(question, default="", completer=None, width=_defaultwidth):
     if (not _readline_available):
@@ -630,6 +656,8 @@ def getpassword(question, width=_defaultwidth, echo=False):
         
         while 1:
             a1 = getpass.getpass(question.ljust(width))
+            if len(a1) == 0:
+                return a1;
             a2 = getpass.getpass("[Repeat] %s" % (question.ljust(width)))
             if a1 == a2:
                 return a1
